/* vim:ft=c ts=4 sw=4 nomodified:
 */
/**@file
 *			Patches for Anthy, by G-HAL
 *@brief	ʸڤꥢ르ꥺࡧʬ
 *@date		Wed,05 Nov,2008
 *@date		Sun,16 Nov,2008
 *@date		Tue,18 Nov,2008
 *@date		Thu,20 Nov,2008
 *@date		Mon,05 Jan,2009
 *@date		Sat,17 Jan,2009
 *@date		Mon,19 Jan,2009
 *@date		Sat,24 Jan,2009, Thu,29 Jan,2009
 *@date		Tue,03 Feb,2009
 *@date		Wed,04 Feb,2009 - Thu,05 Feb,2009, Sun,08 Feb,2009
 *@date		Fri,20 Feb,2009, Sat,21 Feb,2009, Tue,24 Feb,2009
 *@date		Fri,06 Mar,2009
 *@date		Sat,25 Apr,2009
 *@date		Fri,01 May,2009
 *@date		Sun,07 Jun,2009
 *@date		Thu,27 Aug,2009
 *@author	Copyright(C)2008-2009 G-HAL
 */
#if defined(HAVE_CONFIG_H)	/* Patched by G-HAL, Sun,07 Jun,2009 */
# include "config.h"
#endif

#include <stdlib.h>
#include <math.h>

#include "anthy/settings.h"
#include "anthy/logger.h"
#include "anthy/record.h"
#include "src-splitter/lattice.h"



/** metaword ɾ׻
 *@param[in]		sc				ϥǡ
 *@param[in]		mw				ʸ
 *@param[in]		prev_mw			ʸ
 *@return							ɾ
 *
 *	Patched by G-HAL
 *		Sun,16 Nov,2008
 *		Thu,20 Nov,2008
 *		Tue,03 Feb,2009
 *		Wed,04 Feb,2009
 *		Sat,21 Feb,2009, Tue,24 Feb,2009
 *		Sat,25 Apr,2009
 */
int calc_metaword_score( const struct splitter_context* const sc,
						struct meta_word* const mw,
						const struct meta_word* const prev_mw )
{
	int	score = 0;
	if (NULL == mw) {
		return 0;
	}

	/* δܷ׻ */
	if (anthy_settings.anthy_mode.lattice.with_cand_score) {
		score = mw->cand_hint_freq;
		if ((anthy_settings.anthy_mode.lattice.with_learnedfreq_score)
			&& (0 < mw->cand_hint_learned_freq)
		) {
			score += (int)(log10( (double)mw->cand_hint_learned_freq ) * 10.0);
		}
	} else {
		score = mw->score;
		if ((anthy_settings.anthy_mode.lattice.with_learnedfreq_score)
			&& (0 < mw->cand_hint_learned_freq)
		) {
			score += (int)(log10( (double)mw->cand_hint_learned_freq ) * 100.0);
		}
	}
	if ((MW_ochaire_min <= mw->type) && (mw->type <= MW_ochaire_max)) {
		score += anthy_settings.anthy_mode.lattice.biasscore_by_ochaire;
	}
	if (anthy_settings.anthy_mode.lattice.with_candstruct_score) {
		anthy_mw_eval( mw, (prev_mw ? prev_mw->seg_class : sc->prev_commit.seg_class), anthy_settings.anthy_mode.lattice.with_cand_corpus );
		score = (int) ((double)score * mw->struct_score / RATIO_BASE);

		/* °ܤ¿ˤϥ򲼤롧Σ */
		if (anthy_settings.anthy_mode.depgraph.score.decrease_biasscore_threshold <= mw->cand_hint_depth_of_dep) {
			score += anthy_settings.anthy_mode.depgraph.score.decrease_biasscore;
		}
		score += (mw->cand_hint_depth_of_dep * anthy_settings.anthy_mode.depgraph.score.decrease_score);
	}
	if (score < 1) {
		score = 1;
	}

	if (anthy_settings.anthy_mode.lattice.with_dep_history) {
		if (-1 == mw->extra_score) {
			mw->extra_score = 0;
			do {
				xstr	xs;
				if (sc->char_count < (mw->from + mw->len - mw->cand_hint_length_of_dep)) {
					anthy_log( 1, "***BUG*** calc_metaword_score() max:%d, from:%d, len:%d, dep-len:%d\n",
							  sc->char_count, mw->from, mw->len, mw->cand_hint_length_of_dep );
					break;
				}
				if (anthy_select_section(DEP_HISTORY,0)) {
					break;
				}
				xs.str = sc->ce[mw->from + mw->len - mw->cand_hint_length_of_dep].c;
				xs.len = mw->cand_hint_length_of_dep;
				if ((0 < xs.len) && !anthy_select_row(&xs,0)) {
					const int	count = anthy_get_row_frequency();
					if (0 < count) {
						mw->extra_score = (int)( (double)anthy_settings.anthy_mode.depgraph.score.increase_biasscore_by_learn
												* log10( 1.0 + count ) );
					}
				}
			} while(0);
		}
		score += mw->extra_score;
	}

	return score;
}



/** metaword ͥ٤׻
 *@param[in]		pri					Ѥͥٽɽ
 *@param[in]		mw					ͥ٤metaword
 *@param			cmp_priority		оݤȤʤ priority
 *@param			cmp_minlen			оݤȤʤʸ
 *@param[out]		have_sub_priority	sub-priority Ĥݤ
 *@param[out]		ret_minlen			Ӵʸоݤ cmp_minlen ȤϤ͡
 *@return								ͥ١礭ۤɹ⤤
 *
 *	(lhs|rhs)->mw->type ˤ MW_OCHAIRE, MW_CANDHISTORY ʳʪ⤢Τա
 *
 *	Patched by G-HAL
 *		Wed,05 Nov,2008
 *		Sun,16 Nov,2008
 *		Tue,18 Nov,2008
 *		Thu,20 Nov,2008
 *		Mon,05 Jan,2009
 *		Sat,17 Jan,2009
 *		Mon,19 Jan,2009
 *		Sat,24 Jan,2009, Thu,29 Jan,2009
 *		Sun,08 Feb,2009
 *		Tue,24 Feb,2009
 *		Fri,01 May,2009
 *		Thu,27 Aug,2009
 */
int calc_cmpnode_priority( const struct priority_of_metaword_t* const pri,
						  struct meta_word* const mw,
						  int cmp_priority,
						  int cmp_minlen,
						  int* const have_sub_priority,
						  int* const ret_minlen )
{
	*ret_minlen = 0;
	switch (mw->type) {
	case MW_OCHAIRE:
		{
			if (mw->score <= 0) {
				*ret_minlen = 0;
				*have_sub_priority = pri->DELETEDHISTORY.sub;
				return pri->DELETEDHISTORY.pri;
			}
			if (NULL == mw->mw1_left) {
				if (NULL == mw->mw1_right) {
					*ret_minlen = mw->len;
					*have_sub_priority = pri->OCHAIRE_SINGLE.sub;
					return pri->OCHAIRE_SINGLE.pri;
				}
				*ret_minlen = mw->ochaire_total_len;
				*have_sub_priority = pri->OCHAIRE_HEAD.sub;
				return pri->OCHAIRE_HEAD.pri;
			}
			if (NULL == mw->mw1_right) {
				*ret_minlen = mw->len;
				*have_sub_priority = pri->OCHAIRE_TAIL.sub;
				return pri->OCHAIRE_TAIL.pri;
			}
			*ret_minlen = mw->ochaire_total_len;
			*have_sub_priority = pri->OCHAIRE_MIDDLE.sub;
			return pri->OCHAIRE_MIDDLE.pri;
		}
		break;
	case MW_OCHAIREwithoutINDEP:
		{
			if (mw->score <= 0) {
				*ret_minlen = 0;
				*have_sub_priority = pri->DELETEDHISTORY.sub;
				return pri->DELETEDHISTORY.pri;
			}
			if (NULL == mw->mw1_left) {
				if (NULL == mw->mw1_right) {
					*ret_minlen = mw->len;
					*have_sub_priority = pri->OCHAIREwoI_SINGLE.sub;
					return pri->OCHAIREwoI_SINGLE.pri;
				}
				*ret_minlen = mw->ochaire_total_len;
				*have_sub_priority = pri->OCHAIREwoI_HEAD.sub;
				return pri->OCHAIREwoI_HEAD.pri;
			}
			if (NULL == mw->mw1_right) {
				*ret_minlen = mw->len;
				*have_sub_priority = pri->OCHAIREwoI_TAIL.sub;
				return pri->OCHAIREwoI_TAIL.pri;
			}
			*ret_minlen = mw->ochaire_total_len;
			*have_sub_priority = pri->OCHAIREwoI_MIDDLE.sub;
			return pri->OCHAIREwoI_MIDDLE.pri;
		}
		break;
	case MW_OCHAIREwithoutDEP:
		{
			if (mw->score <= 0) {
				*ret_minlen = 0;
				*have_sub_priority = pri->DELETEDHISTORY.sub;
				return pri->DELETEDHISTORY.pri;
			}
			if (NULL == mw->mw1_left) {
				if (NULL == mw->mw1_right) {
					*ret_minlen = mw->len;
					*have_sub_priority = pri->OCHAIREwoD_SINGLE.sub;
					return pri->OCHAIREwoD_SINGLE.pri;
				}
				*ret_minlen = mw->ochaire_total_len;
				*have_sub_priority = pri->OCHAIREwoD_HEAD.sub;
				return pri->OCHAIREwoD_HEAD.pri;
			}
			if (NULL == mw->mw1_right) {
				*ret_minlen = mw->len;
				*have_sub_priority = pri->OCHAIREwoD_TAIL.sub;
				return pri->OCHAIREwoD_TAIL.pri;
			}
			*ret_minlen = mw->ochaire_total_len;
			*have_sub_priority = pri->OCHAIREwoD_MIDDLE.sub;
			return pri->OCHAIREwoD_MIDDLE.pri;
		}
		break;
	case MW_OCHAIREwithoutINDEPwithoutDEP:
		{
			if (mw->score <= 0) {
				*ret_minlen = 0;
				*have_sub_priority = pri->DELETEDHISTORY.sub;
				return pri->DELETEDHISTORY.pri;
			}
			if (NULL == mw->mw1_left) {
				if (NULL == mw->mw1_right) {
					*ret_minlen = mw->len;
					*have_sub_priority = pri->OCHAIREwoIwoD_SINGLE.sub;
					return pri->OCHAIREwoIwoD_SINGLE.pri;
				}
				*ret_minlen = mw->ochaire_total_len;
				*have_sub_priority = pri->OCHAIREwoIwoD_HEAD.sub;
				return pri->OCHAIREwoIwoD_HEAD.pri;
			}
			if (NULL == mw->mw1_right) {
				*ret_minlen = mw->len;
				*have_sub_priority = pri->OCHAIREwoIwoD_TAIL.sub;
				return pri->OCHAIREwoIwoD_TAIL.pri;
			}
			*ret_minlen = mw->ochaire_total_len;
			*have_sub_priority = pri->OCHAIREwoIwoD_MIDDLE.sub;
			return pri->OCHAIREwoIwoD_MIDDLE.pri;
		}
		break;
	case MW_CANDHISTORY:
		{
			if (mw->score <= 0) {
				*ret_minlen = 0;
				*have_sub_priority = pri->DELETEDHISTORY.sub;
				return pri->DELETEDHISTORY.pri;
			}
			if ((pri->CAND_COUNTER_MINLIMIT.pri <= cmp_priority) && (cmp_minlen <= mw->len)) {
				if (pri->CAND_COUNTER_MAXLIMIT.pri < cmp_priority) {
					cmp_priority = pri->CAND_COUNTER_MAXLIMIT_VALUE.pri;
				}
				if (mw->ochaire_opposite < cmp_priority) {
					mw->ochaire_opposite = cmp_priority;
				}
			}
			if (0 < mw->ochaire_opposite) {
				*ret_minlen = mw->len;
				*have_sub_priority = pri->CAND_COUNTER_MINLIMIT.sub;
				return mw->ochaire_opposite;
			}
			if (anthy_settings.anthy_mode.lattice.with_candhistory) {
				*ret_minlen = mw->len;
				*have_sub_priority = pri->CAND_STRONGMODE.sub;
				return pri->CAND_STRONGMODE.pri;
			}
			*have_sub_priority = pri->CAND_WEAKMODE.sub;
			return pri->CAND_WEAKMODE.pri;
		}
		break;
	case MW_COMPOUND_HEAD:
		{
			*have_sub_priority = pri->COMPOUNDHEAD.sub;
			return pri->COMPOUNDHEAD.pri;
		}
		break;
	case MW_COMPOUND_PART:
		{
			*have_sub_priority = pri->COMPOUNDPART.sub;
			return pri->COMPOUNDPART.pri;
		}
		break;
	case MW_COMPOUND:
		{
			*have_sub_priority = pri->COMPOUND.sub;
			return pri->COMPOUND.pri;
		}
		break;
	case MW_COMPOUND_LEAF:
		{
			*have_sub_priority = pri->COMPOUNDLEAF.sub;
			return pri->COMPOUNDLEAF.pri;
		}
		break;
	default:
		*have_sub_priority = pri->DEFAULT.sub;
		return pri->DEFAULT.pri;
		break;
	}
	*have_sub_priority = pri->DELETEDHISTORY.sub;
	return pri->DELETEDHISTORY.pri;
}

/* [ End of File ] */
